;
; RAW DISK DUMP UTILITY FOR C1P/UK101
;
; DUMPS THE CONTENTS OF A DISK AND SENDS OUTPUT
; TO THE CASSETTE PORT AS ASCII HEX. IT WILL
; CAPTURE ALL DATA ON THE DISK INCLUDING THE
; TRACK/SECTOR FORMATTING INFORMATION.
;
; DUMPS EVERY BYTE ENCOUNTERED ON A TRACK
; STARTING AT THE INDEX PULSE.
;
; NOTE: DOS FORMATTED TRACKS WILL CONTAIN
; SPURIOUS CHARACTERS BEFORE AND AFTER SECTORS
; AND CAN BE IGNORED.
;
;
; HISTORY:
;
; 0.4  CODE REVAMPED. DISABLED ERROR CHECK
;      (USELESS FOR DOS DISKS). REMOVED
;      MESSAGES FROM DUMP OUTPUT.
;
; 0.3  INCREASED DELAYS FOR USE ON FASTER
;      MACHINES.
;
; 0.21 FIXED ERROR COUNT. CHANGED BUFFER
;      BUFFER SIZE TO 9 PAGES AND FILL CHAR
;      TO $00.
;
; 0.2  ACIA IS RESET BEFORE EACH TRACK READ.
;      PARITY ERRORS CHECKED. READING STOPS
;      AT THE NEXT INDEX PULSE, SO NO MORE
;      EXTRANEOUS DATA IN TRACK DUMPS.
;
; 0.1  1ST RELEASE
;

ORG	=	$0400

MAXTRK	=	40	; # TRACKS TO READ
MAXPAG	=	9	; # PAGES TO READ
FILLCH	=	$00	; FILL CHARACTER

BUFFER	=	$4000	; TRACK BUFFER

; OSI MONITOR ROM ROUTINES

LOADF	=	$203	; LOAD FLAG
SAVEF	=	$205	; SAVE FLAG
INPUT	=	$FFEB	; GET CHAR FROM KEYBOARD/ACIA
OUTPUT	=	$FFEE	; PUT CHAR TO SCREEN/ACIA

DISK	=	$C000	; DISK CONTROLLER

CR	=	$0D
LF	=	$0A

	*=	ORG

	JSR	SAVEOFF

	JSR	PRINT
	.BYTE	CR,LF
	.BYTE	'RAW DISK DUMP 0.4',CR,LF,CR,LF
	.BYTE	'DUMP RAW TRACK DATA FROM',CR,LF
	.BYTE	'DISKS TO CASSETTE',CR,LF,CR,LF,0

	JSR	PRINT
	.BYTE	'WRITE PROTECT ALL DISKS!',CR,LF
	.BYTE	CR,LF
	.BYTE	'INSERT DISK TO BE DUMPED',0

	JSR	ANYKEY

	JSR	INITPIA
	JSR	RESERR

	LDA	#0
	STA	TRK
LOOP	LDA	TRK
	CMP	#MAXTRK
	BEQ	QUIT

	PHA
	JSR	READMSG
	PLA
	BNE	LOOP2

	JSR	TZERO	; STEP TO TRACK 0
	JMP	LOOP3

LOOP2	JSR	TNEXT	; STEP TO NEXT TRACK

LOOP3	JSR	RDTRK
	JSR	SAVMSG
	LDA	#MAXPAG
	JSR	SAVBUF
	INC	TRK
	JMP	LOOP

QUIT
;	JSR	SHOWERR
	JSR	PRINT
	.BYTE	CR,LF,CR,LF
	.BYTE	'DONE ... REMOVE DISK',CR,LF,0
	JSR	INPUT
	JMP	($FFFC)

; SAVE BUFFER CONTENTS (A)

SAVBUF	PHA
	JSR	RESPTR
	JSR	SAVEON
	PLA
	STA	$FF
	LDA	#0
	STA	CNT
	STA	CNT+1
	TAY
	TAX
	BEQ	SAVBUF5

SAVBUF1	CPX	#0
	BNE	SAVBUF2
	JSR	CRLF
	LDX	#16
	BNE	SAVBUF3

SAVBUF2	LDA	#' '
	JSR	OUTPUT
SAVBUF3	LDA	($FD),Y
	JSR	PHEXA
	INC	CNT
	BNE	SAVBUF4
	INC	CNT+1
SAVBUF4	DEX
	INY
	BNE	SAVBUF1
	INC	$FE
	DEC	$FF
SAVBUF5	LDA	$FF
	BNE	SAVBUF1
	JSR	CRLF
	JMP	SAVEOFF

; INIT POINTERS, CLEAR BUFFER

INIT	JSR	RESPTR	; SET POINTERS
	LDY	#0
	LDA	#FILLCH
INIT1	STA	($FD),Y
	INY
	BNE	INIT1
	INC	$FE
	DEC	$FF
	BNE	INIT1
	JMP	RESPTR

; RESET BUFFER POINTERS

RESPTR	LDA	#MAXPAG
	STA	$FF
	LDA	#<BUFFER
	STA	$FD
	LDA	#>BUFFER
	STA	$FE
	RTS

; INIT DISK CONTROLLER PIA

INITPIA	LDY	#0
	STY	DISK+1	; SELECT DDRA
	STY	DISK	; SET PORTA TO ALL INPUTS
	LDX	#4	; 0000 0100
	STX	DISK+1	; SELECT PORTA
	STY	DISK+3	; SELECT DDRB
	DEY
	STY	DISK+2	; SET PORTB TO ALL OUTPUTS
	STX	DISK+3	; SELECT PORTB
	STY	DISK+2	; SET PORTB OUTPUTS HIGH
	RTS

; STEP TO TRACK 0

TZERO	LDA	#$FB	; 1111 1011
	BNE	TZERO3	; DIR=INWARDS

TZERO1	LDA	#2	; 0000 0010
	BIT	DISK	; TEST 'TRK0' PIN
	BNE	TZERO2	; AT TRACK 0
	RTS

TZERO2	LDA	#$FF	; DIR=OUTWARDS (TO TRK0)

TZERO3	STA	DISK+2	; SET 'DIR' PIN
	JSR	DELAY2
	AND	#$F7	; 1111 0111
	STA	DISK+2	; SET 'STEP' PIN LOW
	JSR	DELAY2
	ORA	#8	; 0000 1000
	STA	DISK+2	; SET 'STEP' PIN HIGH
	LDX	#$30	; 60 MS
	JSR	DELAY
	BEQ	TZERO1	; ALWAYS

; STEP TO NEXT TRACK

TNEXT	LDA	#$FB	; 1111 1011
			; DIR=INWARDS
	STA	DISK+2	; SET 'DIR' PIN
	JSR	DELAY2
	AND	#$F7	; 1111 0111
	STA	DISK+2	; SET 'STEP' LOW
	JSR	DELAY2
	ORA	#8	; 0000 1000
	STA	DISK+2	; SET 'STEP' HIGH
	LDX	#$30	; 60MS
	JMP	DELAY

; READ CURRENT TRACK TO BUFFER

RDTRK	JSR	INIT
	LDX	#$7F	; 0111 1111
	STX	DISK+2	; LOAD DISK HEAD
	LDX	#$FF	; 320MS
	JSR	DELAY
RDTRK1	LDA	DISK	; WAIT START OF
	BMI	RDTRK1	; INDEX PULSE
RDTRK2	LDA	DISK	; WAIT END OF
	BPL	RDTRK2	; INDEX PULSE
	LDA	#3	; RESET ACIA
	STA	DISK+$10
	LDA	#$58	; DIVIDE BY 1, 8E1, RTS HIGH
	STA	DISK+$10
	LDY	#0	; READ UNTIL ALL PAGES DONE
RDTRK3	JSR	RDBYT
	BCS	RDTRK4	; OR END-OF-TRACK
	STA	($FD),Y
	INY
	BNE	RDTRK3
	INC	$FE
	DEC	$FF
	BNE	RDTRK3
RDTRK4	LDA	#$FF
	STA	DISK+2	; UNLOAD DISK HEAD
	RTS

; DELAY 1.25MS PER LOOP

DELAY	LDY	#$F8
DELAY1	DEY
	BNE	DELAY1
	EOR	$FF,X
	DEX
	BNE	DELAY
	RTS

DELAY2	JSR	DELAY21
DELAY21	RTS

; INPUT CHAR FROM DISK, CY SET IF END-OF-TRACK

RDBYT	LDA	DISK	; TEST FOR INDEX PULSE
	BPL	RDBYT2
	LDA	DISK+$10
	LSR	A
	BCC	RDBYT
	BIT	DISK+$10 ; TEST PARITY ERROR
	BVS	RDBYT3
	LDA	DISK+$11 ; READ BYTE
	CLC
	RTS

RDBYT2	LDA	#FILLCH
	SEC
	RTS

RDBYT3	INC	ERRCNT
	BNE	RDBYT4
	INC	ERRCNT+1
RDBYT4	LDA	DISK+$11 ; READ BYTE
	CLC
	RTS

; PRINT IN-LINE STRING

PRINT	PLA
	STA	$FD
	PLA
	STA	$FE
	LDY	#1
PRINT1	LDA	($FD),Y
	BEQ	PRINT2
	JSR	OUTPUT
	INY
	BNE	PRINT1
PRINT2	TYA
	SEC
	ADC	$FD
	STA	$FD
	BCC	PRINT3
	INC	$FE
PRINT3	JMP	($FD)

; PRINT HEX WORD (A,X)

PHEX	JSR	PHEXA
	TXA

; PRINT HEX BYTE (A)

PHEXA	PHA
	LSR	A
	LSR	A
	LSR	A
	LSR	A
	JSR	PHEXA1
	PLA
PHEXA1	AND	#$F
	ORA	#'0'
	CMP	#'9'+1
	BMI	PHEXA2
	CLC
	ADC	#7
PHEXA2	JMP	OUTPUT

; PRINT DECIMAL (A)

PDECA	TAX
	LDA	#0

; PRINT DECIMAL (A,X)

PDEC	STX	PDEC8
	STA	PDEC8+1
	LDA	#0
	LDX	#2
PDEC1	STA	PDEC10,X
	DEX
	BPL	PDEC1
	LDY	#16
	PHP
	SEI
	SED
PDEC2	ASL	PDEC8
	ROL	PDEC8+1
	LDX	#2
PDEC3	LDA	PDEC10,X
	ADC	PDEC10,X
	STA	PDEC10,X
	DEX
	BPL	PDEC3
	DEY
	BNE	PDEC2
	PLP
	LDX	#6
	LDA	#0
	STA	PDEC9
PDEC4	LDY	#3
	LDA	#0
PDEC5	ASL	PDEC10+2
	ROL	PDEC10+1
	ROL	PDEC10
	ROL	A
	DEY
	BPL	PDEC5
	TAY
	BNE	PDEC6
	CPX	#1
	BEQ	PDEC6
	LDY	PDEC9
	BEQ	PDEC7
PDEC6	INC	PDEC9
	ORA	#$30
	JSR	OUTPUT
PDEC7	DEX	
	BNE	PDEC4
	RTS

PDEC8	.WORD	0
PDEC9	.BYTE	0
PDEC10	.BYTE	0,0,0

CRLF	LDA	#CR
	JSR	OUTPUT
	LDA	#LF
	JMP	OUTPUT

ANYKEY	JSR	PRINT
	.BYTE	CR,LF
	.BYTE	'PRESS ANY KEY WHEN READY',CR,LF,0
	JMP	INPUT

READMSG	PHA
	JSR	PRINT
	.BYTE	CR,LF,CR,LF
	.BYTE	'READING TRACK ',0
	PLA
	JMP	PDECA

SAVMSG	JSR	PRINT
	.BYTE	' ...SAVING',CR,LF,0
	RTS

SHOWERR	JSR	PRINT
	.BYTE	CR,LF,CR,LF,'ERRORS ',0
	LDA	ERRCNT+1
	LDX	ERRCNT
	JSR	PDEC
	JMP	CRLF

SAVEON	LDA	#1
	STA	SAVEF
	RTS

SAVEOFF	LDA	#0
	STA	LOADF
	STA	SAVEF
	RTS

RESERR	LDA	#0
	STA	ERRCNT
	STA	ERRCNT+1
	RTS

CNT	.WORD	0
ERRCNT	.WORD	0
TRK	.BYTE	0

	.END
